==> flow-maven-plugin-2.7.17-sources.jar: com/vaadin/flow/plugin/maven/PrepareFrontendMojo.java <==
/*
 * Copyright 2000-2020 Vaadin Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.vaadin.flow.plugin.maven;

import java.io.File;
import java.io.IOException;
import java.io.UncheckedIOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.io.FileUtils;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.plugins.annotations.Component;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.apache.maven.plugins.annotations.ResolutionScope;
import org.apache.maven.project.MavenProject;
import org.sonatype.plexus.build.incremental.BuildContext;

import com.vaadin.flow.server.Constants;
import com.vaadin.flow.server.ExecutionFailedException;
import com.vaadin.flow.server.frontend.FrontendTools;
import com.vaadin.flow.server.frontend.FrontendUtils;
import com.vaadin.flow.server.frontend.NodeTasks;

import elemental.json.Json;
import elemental.json.JsonObject;
import elemental.json.impl.JsonUtil;
import static com.vaadin.flow.plugin.common.FlowPluginFrontendUtils.getClassFinder;
import static com.vaadin.flow.server.Constants.FRONTEND_TOKEN;
import static com.vaadin.flow.server.Constants.GENERATED_TOKEN;
import static com.vaadin.flow.server.Constants.NPM_TOKEN;
import static com.vaadin.flow.server.Constants.SERVLET_PARAMETER_COMPATIBILITY_MODE;
import static com.vaadin.flow.server.Constants.SERVLET_PARAMETER_PRODUCTION_MODE;
import static com.vaadin.flow.server.InitParameters.NODE_DOWNLOAD_ROOT;
import static com.vaadin.flow.server.InitParameters.NODE_VERSION;
import static com.vaadin.flow.server.frontend.FrontendUtils.FRONTEND;
import static com.vaadin.flow.server.frontend.FrontendUtils.TOKEN_FILE;

/**
 * This goal checks that node and npm tools are installed, copies frontend
 * resources available inside `.jar` dependencies to `node_modules`, and creates
 * or updates `package.json` and `webpack.config.json` files.
 *
 * @since 2.0
 */
@Mojo(name = "prepare-frontend", requiresDependencyResolution = ResolutionScope.COMPILE_PLUS_RUNTIME, defaultPhase = LifecyclePhase.PROCESS_RESOURCES)
public class PrepareFrontendMojo extends FlowModeAbstractMojo {

    /**
     * This goal checks that node and npm tools are installed, copies frontend
     * resources available inside `.jar` dependencies to `node_modules`, and
     * creates or updates `package.json` and `webpack.config.json` files.
     *
     * @deprecated use {@link PrepareFrontendMojo} instead
     */
    @Deprecated
    @Mojo(name = "validate", requiresDependencyResolution = ResolutionScope.COMPILE_PLUS_RUNTIME, defaultPhase = LifecyclePhase.PROCESS_RESOURCES)
    public static class VaildateMojo extends PrepareFrontendMojo {
        @Override
        public void execute()
                throws MojoExecutionException, MojoFailureException {
            getLog().warn(
                    "\n\n   You are using the 'validate' goal which has been renamed to 'prepare-frontend', please update your 'pom.xml'.\n");
            super.execute();
        }
    }

    @Parameter(defaultValue = "${project}", readonly = true, required = true)
    private MavenProject project;

    /**
     * The folder where `package.json` file is located. Default is project root
     * dir.
     */
    @Parameter(defaultValue = "${project.basedir}")
    private File npmFolder;

    /**
     * Copy the `webapp.config.js` from the specified URL if missing. Default is
     * the template provided by this plugin. Set it to empty string to disable
     * the feature.
     */
    @Parameter(defaultValue = FrontendUtils.WEBPACK_CONFIG)
    private String webpackTemplate;

    /**
     * Copy the `webapp.generated.js` from the specified URL. Default is the
     * template provided by this plugin. Set it to empty string to disable the
     * feature.
     */
    @Parameter(defaultValue = FrontendUtils.WEBPACK_GENERATED)
    private String webpackGeneratedTemplate;

    /**
     * The folder where flow will put generated files that will be used by
     * webpack.
     */
    @Parameter(defaultValue = "${project.build.directory}/" + FRONTEND)
    private File generatedFolder;

    @Component
    private BuildContext buildContext; // m2eclipse integration

    @Override
    public void execute() throws MojoExecutionException, MojoFailureException {
        super.execute();

        // propagate info via System properties and token file
        propagateBuildInfo();

        // Do nothing when compatibility mode
        if (compatibility) {
            getLog().debug(
                    "Skipped 'prepare-frontend' goal because compatibility mode is set.");
            return;
        }

        final URI nodeDownloadRootURI;
        try {
            nodeDownloadRootURI = new URI(nodeDownloadRoot);
        } catch (URISyntaxException e) {
            throw new MojoExecutionException("Failed to parse " + nodeDownloadRoot, e);
        }
        try {
            FrontendTools tools = new FrontendTools(npmFolder.getAbsolutePath(),
                    () -> FrontendUtils.getVaadinHomeDirectory().getAbsolutePath(),
                    nodeVersion, nodeDownloadRootURI, requireHomeNodeExec);
            tools.validateNodeAndNpmVersion();
        } catch (IllegalStateException exception) {
            throw new MojoExecutionException(exception.getMessage(), exception);
        }
        try {
            FileUtils.forceMkdir(generatedFolder);
        } catch (IOException e) {
            throw new MojoFailureException(
                    "Failed to create folder '" + generatedFolder
                            + "'. Verify that you may write to path.", e);
        }
        try {
            NodeTasks.Builder builder = new NodeTasks.Builder(getClassFinder(project), npmFolder,
                    generatedFolder, frontendDirectory)
                            .withWebpack(webpackOutputDirectory,
                                    webpackTemplate, webpackGeneratedTemplate)
                            .createMissingPackageJson(true)
                            .enableImportsUpdate(false)
                            .enablePackagesUpdate(false)
                            .runNpmInstall(false)
                            .withNodeVersion(nodeVersion)
                            .withNodeDownloadRoot(nodeDownloadRootURI)
                            .withHomeNodeExecRequired(requireHomeNodeExec);
            // If building a jar project copy jar artifact contents now as we might
            // not be able to read files from jar path.
            if("jar".equals(project.getPackaging())) {
                Set<File> jarFiles = project.getArtifacts().stream()
                        .filter(artifact -> "jar".equals(artifact.getType()))
                        .map(Artifact::getFile).collect(Collectors.toSet());
                builder.copyResources(jarFiles);
            }

            builder.build().execute();
        } catch (ExecutionFailedException exception) {
            throw new MojoFailureException(
                    "Could not execute prepare-frontend goal.", exception);
        }

    }

    private void propagateBuildInfo() {
        // For forked processes not accessing to System.properties we leave a
        // token file with the information about the build
        File token = new File(webpackOutputDirectory, TOKEN_FILE);
        JsonObject buildInfo = Json.createObject();
        buildInfo.put(SERVLET_PARAMETER_COMPATIBILITY_MODE, compatibility);
        buildInfo.put(SERVLET_PARAMETER_PRODUCTION_MODE, productionMode);
        buildInfo.put(NPM_TOKEN, npmFolder.getAbsolutePath());
        buildInfo.put(GENERATED_TOKEN, generatedFolder.getAbsolutePath());
        buildInfo.put(FRONTEND_TOKEN, frontendDirectory.getAbsolutePath());
        buildInfo.put(Constants.SERVLET_PARAMETER_ENABLE_PNPM, pnpmEnable);
        buildInfo.put(Constants.REQUIRE_HOME_NODE_EXECUTABLE,
                requireHomeNodeExec);
        buildInfo.put(NODE_VERSION, nodeVersion);
        buildInfo.put(NODE_DOWNLOAD_ROOT, nodeDownloadRoot);

        try {
            FileUtils.forceMkdir(token.getParentFile());
            FileUtils.write(token, JsonUtil.stringify(buildInfo, 2) + "\n",
                    StandardCharsets.UTF_8.name());

            // Inform m2eclipse that the directory containing the token file has
            // been updated in order to trigger server re-deployment (#6103)
            if (buildContext != null) {
                buildContext.refresh(token.getParentFile());
            }
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }

        // Enable debug to find out problems related with flow modes
        Log log = getLog();
        if (log.isDebugEnabled()) {
            log.debug(String.format(
                    "%n>>> Running prepare-frontend in %s project%nSystem"
                            + ".properties:%n productionMode: %s%n bowerMode:"
                            + " %s%n compatibilityMode: %s%n webpackPort: %s%n "
                            + "project.basedir: %s%nGoal parameters:%n "
                            + "productionMode: %s%n compatibilityMode: %s%n "
                            + "compatibility: %b%n npmFolder: %s%nToken file: "
                            + "%s%n" + "Token content: %s%n",
                    project.getName(),
                    System.getProperty("vaadin.productionMode"),
                    System.getProperty("vaadin.bowerMode"),
                    System.getProperty("vaadin.compatibiityMode"),
                    System.getProperty("vaadin.devmode.webpack.running-port"),
                    System.getProperty("project.basedir"), productionMode,
                    compatibilityMode, compatibility, npmFolder,
                    token.getAbsolutePath(), buildInfo.toJson()));
        }
    }

    @Override
    boolean isDefaultCompatibility() {
        return false;
    }
}

==> flow-maven-plugin-2.7.23-sources.jar: com/vaadin/flow/plugin/maven/PrepareFrontendMojo.java <==
/*
 * Copyright 2000-2020 Vaadin Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.vaadin.flow.plugin.maven;

import java.io.File;
import java.io.IOException;
import java.io.UncheckedIOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.io.FileUtils;
import org.apache.maven.artifact.Artifact;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.plugins.annotations.Component;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.apache.maven.plugins.annotations.ResolutionScope;
import org.apache.maven.project.MavenProject;
import org.sonatype.plexus.build.incremental.BuildContext;

import com.vaadin.flow.server.Constants;
import com.vaadin.flow.server.ExecutionFailedException;
import com.vaadin.flow.server.frontend.FrontendTools;
import com.vaadin.flow.server.frontend.FrontendUtils;
import com.vaadin.flow.server.frontend.NodeTasks;
import com.vaadin.flow.server.frontend.installer.Platform;

import elemental.json.Json;
import elemental.json.JsonObject;
import elemental.json.impl.JsonUtil;
import static com.vaadin.flow.plugin.common.FlowPluginFrontendUtils.getClassFinder;
import static com.vaadin.flow.server.Constants.FRONTEND_TOKEN;
import static com.vaadin.flow.server.Constants.GENERATED_TOKEN;
import static com.vaadin.flow.server.Constants.NPM_TOKEN;
import static com.vaadin.flow.server.Constants.SERVLET_PARAMETER_COMPATIBILITY_MODE;
import static com.vaadin.flow.server.Constants.SERVLET_PARAMETER_PRODUCTION_MODE;
import static com.vaadin.flow.server.InitParameters.NODE_DOWNLOAD_ROOT;
import static com.vaadin.flow.server.InitParameters.NODE_VERSION;
import static com.vaadin.flow.server.frontend.FrontendUtils.FRONTEND;
import static com.vaadin.flow.server.frontend.FrontendUtils.TOKEN_FILE;

/**
 * This goal checks that node and npm tools are installed, copies frontend
 * resources available inside `.jar` dependencies to `node_modules`, and creates
 * or updates `package.json` and `webpack.config.json` files.
 *
 * @since 2.0
 */
@Mojo(name = "prepare-frontend", requiresDependencyResolution = ResolutionScope.COMPILE_PLUS_RUNTIME, defaultPhase = LifecyclePhase.PROCESS_RESOURCES)
public class PrepareFrontendMojo extends FlowModeAbstractMojo {

    /**
     * This goal checks that node and npm tools are installed, copies frontend
     * resources available inside `.jar` dependencies to `node_modules`, and
     * creates or updates `package.json` and `webpack.config.json` files.
     *
     * @deprecated use {@link PrepareFrontendMojo} instead
     */
    @Deprecated
    @Mojo(name = "validate", requiresDependencyResolution = ResolutionScope.COMPILE_PLUS_RUNTIME, defaultPhase = LifecyclePhase.PROCESS_RESOURCES)
    public static class VaildateMojo extends PrepareFrontendMojo {
        @Override
        public void execute()
                throws MojoExecutionException, MojoFailureException {
            getLog().warn(
                    "\n\n   You are using the 'validate' goal which has been renamed to 'prepare-frontend', please update your 'pom.xml'.\n");
            super.execute();
        }
    }

    @Parameter(defaultValue = "${project}", readonly = true, required = true)
    private MavenProject project;

    /**
     * The folder where `package.json` file is located. Default is project root
     * dir.
     */
    @Parameter(defaultValue = "${project.basedir}")
    private File npmFolder;

    /**
     * Copy the `webapp.config.js` from the specified URL if missing. Default is
     * the template provided by this plugin. Set it to empty string to disable
     * the feature.
     */
    @Parameter(defaultValue = FrontendUtils.WEBPACK_CONFIG)
    private String webpackTemplate;

    /**
     * Copy the `webapp.generated.js` from the specified URL. Default is the
     * template provided by this plugin. Set it to empty string to disable the
     * feature.
     */
    @Parameter(defaultValue = FrontendUtils.WEBPACK_GENERATED)
    private String webpackGeneratedTemplate;

    /**
     * The folder where flow will put generated files that will be used by
     * webpack.
     */
    @Parameter(defaultValue = "${project.build.directory}/" + FRONTEND)
    private File generatedFolder;

    @Component
    private BuildContext buildContext; // m2eclipse integration

    @Override
    public void execute() throws MojoExecutionException, MojoFailureException {
        super.execute();

        if(nodeDownloadRoot == null) {
            nodeDownloadRoot = Platform.guess().getNodeDownloadRoot();
        }

        // propagate info via System properties and token file
        propagateBuildInfo();

        // Do nothing when compatibility mode
        if (compatibility) {
            getLog().debug(
                    "Skipped 'prepare-frontend' goal because compatibility mode is set.");
            return;
        }

        final URI nodeDownloadRootURI;
        try {
            nodeDownloadRootURI = new URI(nodeDownloadRoot);
        } catch (URISyntaxException e) {
            throw new MojoExecutionException("Failed to parse " + nodeDownloadRoot, e);
        }
        try {
            FrontendTools tools = new FrontendTools(npmFolder.getAbsolutePath(),
                    () -> FrontendUtils.getVaadinHomeDirectory().getAbsolutePath(),
                    nodeVersion, nodeDownloadRootURI, requireHomeNodeExec);
            tools.validateNodeAndNpmVersion();
        } catch (IllegalStateException exception) {
            throw new MojoExecutionException(exception.getMessage(), exception);
        }
        try {
            FileUtils.forceMkdir(generatedFolder);
        } catch (IOException e) {
            throw new MojoFailureException(
                    "Failed to create folder '" + generatedFolder
                            + "'. Verify that you may write to path.", e);
        }
        try {
            NodeTasks.Builder builder = new NodeTasks.Builder(getClassFinder(project), npmFolder,
                    generatedFolder, frontendDirectory)
                            .withWebpack(webpackOutputDirectory,
                                    webpackTemplate, webpackGeneratedTemplate)
                            .createMissingPackageJson(true)
                            .enableImportsUpdate(false)
                            .enablePackagesUpdate(false)
                            .runNpmInstall(false)
                            .withNodeVersion(nodeVersion)
                            .withNodeDownloadRoot(nodeDownloadRootURI)
                            .withHomeNodeExecRequired(requireHomeNodeExec);
            // If building a jar project copy jar artifact contents now as we might
            // not be able to read files from jar path.
            if("jar".equals(project.getPackaging())) {
                Set<File> jarFiles = project.getArtifacts().stream()
                        .filter(artifact -> "jar".equals(artifact.getType()))
                        .map(Artifact::getFile).collect(Collectors.toSet());
                builder.copyResources(jarFiles);
            }

            builder.build().execute();
        } catch (ExecutionFailedException exception) {
            throw new MojoFailureException(
                    "Could not execute prepare-frontend goal.", exception);
        }

    }

    private void propagateBuildInfo() {
        // For forked processes not accessing to System.properties we leave a
        // token file with the information about the build
        File token = new File(webpackOutputDirectory, TOKEN_FILE);
        JsonObject buildInfo = Json.createObject();
        buildInfo.put(SERVLET_PARAMETER_COMPATIBILITY_MODE, compatibility);
        buildInfo.put(SERVLET_PARAMETER_PRODUCTION_MODE, productionMode);
        buildInfo.put(NPM_TOKEN, npmFolder.getAbsolutePath());
        buildInfo.put(GENERATED_TOKEN, generatedFolder.getAbsolutePath());
        buildInfo.put(FRONTEND_TOKEN, frontendDirectory.getAbsolutePath());
        buildInfo.put(Constants.SERVLET_PARAMETER_ENABLE_PNPM, pnpmEnable);
        buildInfo.put(Constants.REQUIRE_HOME_NODE_EXECUTABLE,
                requireHomeNodeExec);
        buildInfo.put(NODE_VERSION, nodeVersion);
        buildInfo.put(NODE_DOWNLOAD_ROOT, nodeDownloadRoot);

        try {
            FileUtils.forceMkdir(token.getParentFile());
            FileUtils.write(token, JsonUtil.stringify(buildInfo, 2) + "\n",
                    StandardCharsets.UTF_8.name());

            // Inform m2eclipse that the directory containing the token file has
            // been updated in order to trigger server re-deployment (#6103)
            if (buildContext != null) {
                buildContext.refresh(token.getParentFile());
            }
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }

        // Enable debug to find out problems related with flow modes
        Log log = getLog();
        if (log.isDebugEnabled()) {
            log.debug(String.format(
                    "%n>>> Running prepare-frontend in %s project%nSystem"
                            + ".properties:%n productionMode: %s%n bowerMode:"
                            + " %s%n compatibilityMode: %s%n webpackPort: %s%n "
                            + "project.basedir: %s%nGoal parameters:%n "
                            + "productionMode: %s%n compatibilityMode: %s%n "
                            + "compatibility: %b%n npmFolder: %s%nToken file: "
                            + "%s%n" + "Token content: %s%n",
                    project.getName(),
                    System.getProperty("vaadin.productionMode"),
                    System.getProperty("vaadin.bowerMode"),
                    System.getProperty("vaadin.compatibiityMode"),
                    System.getProperty("vaadin.devmode.webpack.running-port"),
                    System.getProperty("project.basedir"), productionMode,
                    compatibilityMode, compatibility, npmFolder,
                    token.getAbsolutePath(), buildInfo.toJson()));
        }
    }

    @Override
    boolean isDefaultCompatibility() {
        return false;
    }
}
